@@ -22,9 +22,9 @@ extern "C" {
  * supports the used interfaces should be chosen by programs that check
  * version information for compatibility. See README for more information. */
 #define LIBJODYCODE_API_VERSION       4
-#define LIBJODYCODE_API_FEATURE_LEVEL 4
-#define LIBJODYCODE_VER               "4.0.1"
-#define LIBJODYCODE_VERDATE           "2025-08-21"
+#define LIBJODYCODE_API_FEATURE_LEVEL 5
+#define LIBJODYCODE_VER               "4.1"
+#define LIBJODYCODE_VERDATE           "2025-09-25"
 #ifdef UNICODE
  #define LIBJODYCODE_WINDOWS_UNICODE  1
 #else
@@ -44,6 +44,7 @@ extern "C" {
  #pragma warning(disable : 4200)
 #endif
 
+
 #include <stdio.h>
 #include <stdint.h>
 #include <sys/types.h>
@@ -77,7 +78,69 @@ extern "C" {
 #endif /* ON_WINDOWS */
 
 
+/* Extend an allocation length to the next 64-bit (8-byte) boundary */
+#define JC_EXTEND64(a) (((a) & 0x7) > 0 ? (((a) & (~0x7)) + 8) : (a))
+
+
+/* File modes and wchar_t type */
+/* Uther things depend on this, keep it at the top */
+#if defined _WIN32 || defined __WIN32 || defined ON_WINDOWS
+ #ifdef UNICODE
+  #define JC_WCHAR_T wchar_t
+  #define JC_FILE_MODE_RDONLY L"rb"
+  #define JC_FILE_MODE_WRONLY L"wb"
+  #define JC_FILE_MODE_RW L"w+b"
+  #define JC_FILE_MODE_RW_EXISTING L"r+b"
+  #define JC_FILE_MODE_WRONLY_APPEND L"ab"
+  #define JC_FILE_MODE_RW_APPEND L"a+b"
+  #define JC_FILE_MODE_RDONLY_SEQ L"rbS"
+  #define JC_FILE_MODE_WRONLY_SEQ L"wbS"
+  #define JC_FILE_MODE_RW_SEQ L"w+bS"
+  #define JC_FILE_MODE_RW_EXISTING_SEQ L"r+bS"
+  #define JC_FILE_MODE_WRONLY_APPEND_SEQ L"abS"
+  #define JC_FILE_MODE_RW_APPEND_SEQ L"a+bS"
+ #else /* Windows, not UNICODE */
+  #define JC_WCHAR_T char
+  #define JC_FILE_MODE_RDONLY "rb"
+  #define JC_FILE_MODE_WRONLY "wb"
+  #define JC_FILE_MODE_RW "w+b"
+  #define JC_FILE_MODE_RW_EXISTING "r+b"
+  #define JC_FILE_MODE_WRONLY_APPEND "ab"
+  #define JC_FILE_MODE_RW_APPEND "a+b"
+  #define JC_FILE_MODE_RDONLY_SEQ "rbS"
+  #define JC_FILE_MODE_WRONLY_SEQ "wbS"
+  #define JC_FILE_MODE_RW_SEQ "w+bS"
+  #define JC_FILE_MODE_RW_EXISTING_SEQ "r+bS"
+  #define JC_FILE_MODE_WRONLY_APPEND_SEQ "abS"
+  #define JC_FILE_MODE_RW_APPEND_SEQ "a+bS"
+ #endif
+ #define JC_F_OK 0
+ #define JC_R_OK 4
+ #define JC_W_OK 2
+ #define JC_X_OK 6
+#else /* Not Windows */
+ #define JC_WCHAR_T char
+ #define JC_FILE_MODE_RDONLY "rb"
+ #define JC_FILE_MODE_WRONLY "wb"
+ #define JC_FILE_MODE_RW "w+b"
+ #define JC_FILE_MODE_RW_EXISTING "r+b"
+ #define JC_FILE_MODE_WRONLY_APPEND "ab"
+ #define JC_FILE_MODE_RW_APPEND "a+b"
+ #define JC_FILE_MODE_RDONLY_SEQ "rb"
+ #define JC_FILE_MODE_WRONLY_SEQ "wb"
+ #define JC_FILE_MODE_RW_SEQ "w+b"
+ #define JC_FILE_MODE_RW_EXISTING_SEQ "r+b"
+ #define JC_FILE_MODE_WRONLY_APPEND_SEQ "ab"
+ #define JC_FILE_MODE_RW_APPEND_SEQ "a+b"
+ #define JC_F_OK F_OK
+ #define JC_R_OK R_OK
+ #define JC_W_OK W_OK
+ #define JC_X_OK X_OK
+#endif /* Windows */
+
+
 /*** time ***/
+/* This is out of order because other things depend on it */
 
 #ifdef ON_WINDOWS
 struct JC_TIMESPEC {
@@ -92,6 +155,7 @@ struct JC_TIMESPEC {
 
 
 /*** stat ***/
+/* This is out of order because other things depend on it */
 
 #ifdef ON_WINDOWS
 struct JC_STAT {
@@ -168,63 +232,57 @@ struct JC_STAT {
  #define JC_S_ISREG(st_mode) S_ISREG(st_mode)
  #define JC_S_ISLNK(st_mode) S_ISLNK(st_mode)
 #endif /* ON_WINDOWS */
-
-#if defined _WIN32 || defined __WIN32 || defined ON_WINDOWS
- #ifdef UNICODE
-  #define JC_WCHAR_T wchar_t
-  #define JC_FILE_MODE_RDONLY L"rb"
-  #define JC_FILE_MODE_WRONLY L"wb"
-  #define JC_FILE_MODE_RW L"w+b"
-  #define JC_FILE_MODE_RW_EXISTING L"r+b"
-  #define JC_FILE_MODE_WRONLY_APPEND L"ab"
-  #define JC_FILE_MODE_RW_APPEND L"a+b"
-  #define JC_FILE_MODE_RDONLY_SEQ L"rbS"
-  #define JC_FILE_MODE_WRONLY_SEQ L"wbS"
-  #define JC_FILE_MODE_RW_SEQ L"w+bS"
-  #define JC_FILE_MODE_RW_EXISTING_SEQ L"r+bS"
-  #define JC_FILE_MODE_WRONLY_APPEND_SEQ L"abS"
-  #define JC_FILE_MODE_RW_APPEND_SEQ L"a+bS"
- #else /* Windows, not UNICODE */
-  #define JC_WCHAR_T char
-  #define JC_FILE_MODE_RDONLY "rb"
-  #define JC_FILE_MODE_WRONLY "wb"
-  #define JC_FILE_MODE_RW "w+b"
-  #define JC_FILE_MODE_RW_EXISTING "r+b"
-  #define JC_FILE_MODE_WRONLY_APPEND "ab"
-  #define JC_FILE_MODE_RW_APPEND "a+b"
-  #define JC_FILE_MODE_RDONLY_SEQ "rbS"
-  #define JC_FILE_MODE_WRONLY_SEQ "wbS"
-  #define JC_FILE_MODE_RW_SEQ "w+bS"
-  #define JC_FILE_MODE_RW_EXISTING_SEQ "r+bS"
-  #define JC_FILE_MODE_WRONLY_APPEND_SEQ "abS"
-  #define JC_FILE_MODE_RW_APPEND_SEQ "a+bS"
- #endif
- #define JC_F_OK 0
- #define JC_R_OK 4
- #define JC_W_OK 2
- #define JC_X_OK 6
-#else /* Not Windows */
- #define JC_WCHAR_T char
- #define JC_FILE_MODE_RDONLY "rb"
- #define JC_FILE_MODE_WRONLY "wb"
- #define JC_FILE_MODE_RW "w+b"
- #define JC_FILE_MODE_RW_EXISTING "r+b"
- #define JC_FILE_MODE_WRONLY_APPEND "ab"
- #define JC_FILE_MODE_RW_APPEND "a+b"
- #define JC_FILE_MODE_RDONLY_SEQ "rb"
- #define JC_FILE_MODE_WRONLY_SEQ "wb"
- #define JC_FILE_MODE_RW_SEQ "w+b"
- #define JC_FILE_MODE_RW_EXISTING_SEQ "r+b"
- #define JC_FILE_MODE_WRONLY_APPEND_SEQ "ab"
- #define JC_FILE_MODE_RW_APPEND_SEQ "a+b"
- #define JC_F_OK F_OK
- #define JC_R_OK R_OK
- #define JC_W_OK W_OK
- #define JC_X_OK X_OK
-#endif /* Windows */
 extern int jc_stat(const char * const filename, struct JC_STAT * const restrict buf);
 
 
+/*** access ***/
+
+extern int jc_access(const char *pathname, int mode);
+
+
+/*** alarm ***/
+
+extern int jc_alarm_ring;
+extern int jc_start_alarm(const unsigned int seconds, const int repeat);
+extern int jc_stop_alarm(void);
+
+
+/*** batch ***/
+
+struct jc_fileinfo {
+	struct JC_STAT *stat;
+	struct JC_DIRENT *dirent;
+	int status;
+};
+
+struct jc_fileinfo_batch {
+	int count;
+	struct jc_fileinfo files[];
+};
+
+extern struct jc_fileinfo_batch *jc_fileinfo_batch_alloc(const int filecnt, const int stat, const int namlen);
+extern void jc_fileinfo_batch_free(struct jc_fileinfo_batch * const restrict batch);
+
+
+/*** cacheinfo ***/
+
+/* Cache information structure
+ * Split caches populate i/d, unified caches populate non-i/d */
+struct jc_proc_cacheinfo {
+	size_t l1;
+	size_t l1i;
+	size_t l1d;
+	size_t l2;
+	size_t l2i;
+	size_t l2d;
+	size_t l3;
+	size_t l3i;
+	size_t l3d;
+};
+
+extern struct jc_proc_cacheinfo *jc_get_proc_cacheinfo(int cleanup);
+
+
 /*** dir ***/
 
 /* Directory stream type
@@ -283,62 +341,10 @@ extern int jc_stat(const char * const filename, struct JC_STAT * const restrict
   #define JC_DT_WHT DT_WHT
  #endif /* DT_UNKNOWN */
 #endif /* ON_WINDOWS */
-extern int jc_closedir(JC_DIR * const restrict dirp);
 extern JC_DIR *jc_opendir(const char * restrict path);
+extern size_t jc_get_d_namlen(const struct JC_DIRENT * const restrict dirent);
 extern struct JC_DIRENT *jc_readdir(JC_DIR * const restrict dirp);
-
-
-extern int        jc_access(const char *pathname, int mode);
-extern char      *jc_getcwd(char * const restrict pathname, const size_t size);
-extern FILE      *jc_fopen(const char *pathname, const JC_WCHAR_T *mode);
-extern int        jc_fclose(FILE *stream);
-extern int        jc_link(const char *path1, const char *path2);
-extern size_t     jc_get_d_namlen(const struct JC_DIRENT * const restrict dirent);
-extern int        jc_rename(const char *oldpath, const char *newpath);
-extern int        jc_remove(const char *pathname);
-
-
-/*** alarm ***/
-
-extern int jc_alarm_ring;
-extern int jc_start_alarm(const unsigned int seconds, const int repeat);
-extern int jc_stop_alarm(void);
-
-
-/*** batch ***/
-
-struct jc_fileinfo {
-	struct JC_STAT *stat;
-	struct JC_DIRENT *dirent;
-	int status;
-};
-
-struct jc_fileinfo_batch {
-	int count;
-	struct jc_fileinfo files[];
-};
-
-extern struct jc_fileinfo_batch *jc_fileinfo_batch_alloc(const int filecnt, const int stat, const int namlen);
-extern void jc_fileinfo_batch_free(struct jc_fileinfo_batch * const restrict batch);
-
-
-/*** cacheinfo ***/
-
-/* Cache information structure
- * Split caches populate i/d, unified caches populate non-i/d */
-struct jc_proc_cacheinfo {
-	size_t l1;
-	size_t l1i;
-	size_t l1d;
-	size_t l2;
-	size_t l2i;
-	size_t l2d;
-	size_t l3;
-	size_t l3i;
-	size_t l3d;
-};
-
-extern struct jc_proc_cacheinfo *jc_get_proc_cacheinfo(int cleanup);
+extern int jc_closedir(JC_DIR * const restrict dirp);
 
 
 /*** error ***/
@@ -350,7 +356,7 @@ extern int jc_print_error(int errnum);
 
 #define JC_ERRORCODE_START 1024
 
-#define JC_ENOERROR   1024
+#define JC_ENOERROR   1024  // 0
 #define JC_ENULL      1025
 #define JC_ECDOTDOT   1026
 #define JC_EGRNEND    1027
@@ -360,32 +366,46 @@ extern int jc_print_error(int errnum);
 #define JC_EALARM     1031
 #define JC_EALLOC     1032
 #define JC_ENUMSTRCMP 1033
-#define JC_EDATETIME  1034
+#define JC_EDATETIME  1034  // 10
 #define JC_EWIN32API  1035
 #define JC_EKERNVER   1036
 #define JC_ENOMEM     1037
+#define JC_ESETVBUF   1038
+
+
+/*** fopen ***/
+
+extern FILE *jc_fopen(const char *pathname, const JC_WCHAR_T *mode);
+extern int jc_fclose(FILE *stream);
 
 
 /*** jc_fwprint ***/
+
 extern int jc_fwprint(FILE * const restrict stream, const char * const restrict str, const int cr);
 
 
+/*** getcwd ***/
+
+extern char *jc_getcwd(char * const restrict pathname, const size_t size);
+
+
 /*** jody_hash ***/
 
-/* Version increments when algorithm changes incompatibly */
-#ifndef JODY_HASH_VERSION
+#ifndef JODY_HASH_H
  #define JODY_HASH_VERSION 7
+ #define JODY_HASH_WIDTH 64
+ typedef uint64_t jodyhash_t;
 #endif
 
-/* Width of a jody_hash */
-#define JODY_HASH_WIDTH 64
-typedef uint64_t jodyhash_t;
-
 enum jc_e_hash { NORMAL = 0, ROLLING = 1 };
-
 extern int jc_block_hash(const enum jc_e_hash type, jodyhash_t *data, jodyhash_t *hash, const size_t count);
 
 
+/*** link ***/
+
+extern int jc_link(const char *path1, const char *path2);
+
+
 /*** linkfiles ***/
 
 enum jc_e_link { SYMLINK = 0, HARDLINK = 1, REFLINK = 2 };
@@ -407,7 +427,18 @@ extern int jc_collapse_dotdot(char * const path);
 extern int jc_make_relative_link_name(const char * const src, const char * const dest, char * rel_path);
 
 
+/*** rename ***/
+
+extern int jc_rename(const char *oldpath, const char *newpath);
+
+
+/*** remove ***/
+
+extern int jc_remove(const char *pathname);
+
+
 /*** size_suffix ***/
+
 /* Suffix definitions (treat as case-insensitive) */
 struct jc_size_suffix {
   const char * const suffix;
@@ -427,17 +458,24 @@ extern int jc_numeric_strcmp(const char * restrict c1, const char * restrict c2,
 
 /*** string ***/
 
-/* string type with length prefix */
-typedef struct _jc_str_t {
-	uint16_t len;
-	char str[1];
-} jc_str_t;
+extern const char *jc_emptystring;
 
+/* string type with length prefix */
+typedef struct _JC_STR_T {
+	uint64_t len;
+	char str[1];
+} JC_STR_T;
+
+extern JC_STR_T *jc_str_init(const char *string, uint32_t len);
 extern int jc_strncaseeq(const char *s1, const char *s2, const size_t len);
 extern int jc_strcaseeq(const char *s1, const char *s2);
 extern int jc_strneq(const char *s1, const char *s2, const size_t len);
 extern int jc_streq(const char *s1, const char *s2);
-extern jc_str_t *jc_str_init(const char * const string, size_t len);
+extern int jc_strteq(const JC_STR_T *s1, const JC_STR_T *s2);
+extern int jc_strtneq(const JC_STR_T *s1, const JC_STR_T *s2, const size_t len);
+extern int jc_strtcaseeq(const JC_STR_T *s1, const JC_STR_T *s2);
+extern int jc_strtncaseeq(const JC_STR_T *s1, const JC_STR_T *s2, const size_t len);
+extern int jc_numeric_strtcmp(const JC_STR_T *c1, const JC_STR_T *c2, const int insensitive);
 
 
 /*** strtoepoch ***/
@@ -461,6 +499,7 @@ extern const int jc_windows_unicode;
  extern int jc_get_kernel_version(void);
 #endif
 
+
 /*** win_unicode ***/
 
 /* Cross-platform help for strings in Unicode mode on Windows
@@ -488,6 +527,8 @@ extern const int jc_windows_unicode;
  #define jc_slash_convert(a)
  #define jc_set_output_modes(a,b)
 #endif
+extern int jc_setup_unicode_terminal(int argc, JC_WCHAR_T **wargv, char ***argv, int *stdout_tty);
+
 
 #ifdef __cplusplus
 }
